import { map, merge, tap } from "rxjs";
class Workspace {
  constructor(config) {
    this.lastDiagnosticChangedEvents = /* @__PURE__ */ new Map();
    this.localizer = config.localizer;
    this.diagnosticProviders = new Map(config.diagnosticProviders?.map((provider) => [provider.id, provider]));
    this.diagnosticsChanged$ = merge(
      ...Array.from(this.diagnosticProviders.values()).map(
        (provider, i) => provider.diagnosticsChanged$.pipe(
          tap((e) => {
            this.updateCombinedDiagnosticChangedEvent(i, e);
          })
        )
      )
    ).pipe(map((e) => this.getCombinedDiagnosticChangedEvent(e.uri, e.version)));
    this.onTypeFormattingProviders = new Map(
      config.onTypeFormattingProviders?.map((provider) => [provider.id, provider])
    );
  }
  async init() {
    await Promise.all(Array.from(this.diagnosticProviders.values()).map((provider) => provider.init()));
    await Promise.all(Array.from(this.onTypeFormattingProviders.values()).map((provider) => provider.init()));
    await this.localizer.init();
  }
  changeLanguage(language) {
    return this.localizer.changeLanguage(language);
  }
  async getDiagnostics(uri) {
    const diagnostics = [];
    for (const provider of this.diagnosticProviders.values()) {
      diagnostics.push(...await provider.getDiagnostics(uri));
    }
    return diagnostics;
  }
  async getDiagnosticFixes(uri, diagnostic) {
    const provider = this.diagnosticProviders.get(diagnostic.source);
    if (provider == null) {
      return [];
    }
    return await provider.getDiagnosticFixes(uri, diagnostic);
  }
  getOnTypeTriggerCharacters() {
    const characters = /* @__PURE__ */ new Set();
    for (const provider of this.onTypeFormattingProviders.values()) {
      for (const ch of provider.onTypeTriggerCharacters) {
        characters.add(ch);
      }
    }
    return Array.from(characters);
  }
  async getOnTypeEdits(uri, position, ch) {
    for (const provider of this.onTypeFormattingProviders.values()) {
      if (provider.onTypeTriggerCharacters.has(ch)) {
        const edits = await provider.getOnTypeEdits(uri, position, ch);
        if (edits != null) {
          return edits;
        }
      }
    }
    return void 0;
  }
  updateCombinedDiagnosticChangedEvent(providerIndex, event) {
    const docEvents = this.lastDiagnosticChangedEvents.get(event.uri) ?? [];
    docEvents[providerIndex] = event;
    this.lastDiagnosticChangedEvents.set(event.uri, docEvents);
  }
  getCombinedDiagnosticChangedEvent(uri, version) {
    const docEvents = this.lastDiagnosticChangedEvents.get(uri) ?? [];
    const diagnostics = [];
    for (const docEvent of docEvents) {
      if (version == null || docEvent.version === version) {
        diagnostics.push(...docEvent.diagnostics);
      }
    }
    return {
      uri,
      version,
      diagnostics
    };
  }
}
export {
  Workspace
};
